#ifndef __midiseq_h__
#define __midiseq_h__

#ifndef __cplusplus
// Whatever uses this stuff must compile as C++
typedef struct midifile midifile;
typedef struct midiseq midiseq;
#else

class midifile {
	private:
		short format, numtracks, ppqn;
	
		unsigned long *tracklength;
		unsigned char **tracks, **trackpos;
		unsigned char *trackflags;

		short curtrack;

		int scan(unsigned char device);

		void setflag(char flag) { trackflags[curtrack] |= flag; }
		void clrflag(char flag) { trackflags[curtrack] &= ~flag; }
	
	protected:
		enum {
			IgnoreTrack = 1,
			IgnorePrograms = 2,
			IgnoreVolumes = 4
		};
		
		char testflag(char flag) const { return (trackflags[curtrack] & flag); }
		int eot() {
			if (curtrack < 0 || curtrack >= numtracks) return 1;
			if ((long)trackpos[curtrack] - (long)tracks[curtrack] >= tracklength[curtrack]) return 1;
			return 0;
		}
		short getbyte(void) { if (eot()) return -1; return *(trackpos[curtrack]++); }
		short peekbyte(unsigned short ahead) {
			if (curtrack < 0 || curtrack >= numtracks) return 1;
			if ((long)trackpos[curtrack]+ahead-(long)tracks[curtrack] >= tracklength[curtrack]) return -1;
			return trackpos[curtrack][ahead];
		}
		unsigned long getpos(void) const { return (long)trackpos[curtrack]-(long)tracks[curtrack]; }
		unsigned char const *getptr(void) const { return trackpos[curtrack]; }
		void setpos(unsigned long pos) { trackpos[curtrack] = (unsigned char*)((long)tracks[curtrack] + pos); }
			
		unsigned long getmidinum();
		unsigned long getbignum(unsigned char *ptr, int size);

		void selecttrack(int track) { curtrack = track; }
		void rewind() { trackpos[curtrack] = tracks[curtrack]; }

	public:
		midifile();
		virtual ~midifile();

		short getnumtracks() const { return numtracks; }
		short getppqn() const { return ppqn; }

		int loadfile(int fp);

		int setdevice(unsigned char dev);
};

class midiseq : public midifile {
	private:
		struct trackdef {
			unsigned long delta;
			unsigned char const *evtptr;
			unsigned char command;
			unsigned long evtlen;
			short loopcount;	// loop repeat counter (-1 = infinite)
			unsigned long loopstart,
			              loopend;
			unsigned char inactive;
		} *seqtrack;
		unsigned char lastcommand;
		int infiniteloop;

		int fetchevt(int track);

	public:
		midiseq();
		~midiseq();

		int setupsequencer(unsigned char dev, int infinite);
		int getevent(unsigned long *delta, unsigned char *command, unsigned char const **evtdata, unsigned long *len);

		void setloop(int loop) { infiniteloop = loop; }
};

#endif


#endif
