#ifndef __buffer_h__
#define __buffer_h__

typedef struct {
	unsigned int samplerate;
	unsigned char channels, bitspersample, filler[2];
	unsigned int numsamples;
	void *data;

	unsigned int albuf;
} PcmBuffer;

jfauderr pcmbuffer_allocate(PcmBuffer *buf, unsigned int numsamples,
		unsigned int samplerate, unsigned char channels, unsigned char bitspersample);
jfauderr pcmbuffer_zero(PcmBuffer *buf);
jfauderr pcmbuffer_free(PcmBuffer *buf);
jfauderr pcmbuffer_freemem(PcmBuffer *buf);
		

typedef struct {
	unsigned int numtracks, ppqn;
	unsigned int *tracklength;
	unsigned char **tracks;
} MidiBuffer;

jfauderr midibuffer_allocate(MidiBuffer *buf, unsigned int numtracks);
jfauderr midibuffer_allocatetrack(MidiBuffer *buf, unsigned int track, unsigned int length);
jfauderr midibuffer_zero(MidiBuffer *buf);
jfauderr midibuffer_free(MidiBuffer *buf);

enum {
	buffertype_pcm,
	buffertype_midi
};

typedef struct {
	int type;
	union {
		PcmBuffer pcm;
		MidiBuffer midi;
	} buf;
} Buffer;

typedef struct BufferLink_t {
	struct BufferLink_t *next,*prev;
	fileident ident;
	int ref, age;
	Buffer buffer;
} BufferLink;

jfauderr buffer_init(void);
jfauderr buffer_uninit(void);
jfauderr buffer_fetch(fileident ident, Buffer **buf);
jfauderr buffer_isavailable(fileident ident, int *type);
jfauderr buffer_release(fileident ident);
jfauderr buffer_manage(fileident ident, Buffer *buf);
jfauderr buffer_update(void);

#endif // __buffer_h__
